"""Manage binding views.

The Views module manages the on-screen keymap representations. A View
is registered for a specific context. The view will be called with a
list of all active bindings whenever the bindings change."""

import Events, Node, curses

class View(Node.Node):
    def __init__(self, context, label, x, y, dx, dy):
        Node.Node.__init__(self)
        self._context = context
        self._label = label
        self._win = None
        self.x, self.y, self.dx, self.dy = x, y, dx, dy
    
    def init(self):
        self._win = self.owner.win().derwin(self.dy, self.dx, self.y, self.x)
        self.drawFrame()

    def drawFrame(self, subtitle=None):
        label = ' %s ' % self._label
        if subtitle:
            label = '%s- %s ' % (label, subtitle)
        self._win.border()
        self._win.addstr(0,2,label)
        self._win.nooutrefresh()
        
    def win(self):
        return self._win

    def updateView(self, bindings):
        raise NotImplementedError
        

class WidgetView(View):

    _widgets = Node.NodeMap()

    def __init__(self, context, label, x, y, dx, dy):
        View.__init__(self, context, label, x, y, dx, dy)

    def init(self):
        View.init(self)
        for widget in self._widgets.itervalues():
            widget.init()
        
    def add(self,widget):
        if self.win() : raise RuntimeError, 'Cannot change active view'
        self._widgets[widget.code] = widget

    def updateView(self, bindings):
        if bindings:
            self.drawFrame(bindings.itervalues().next().keymap().keylist().path())
        for code, widget in self._widgets.iteritems():
            event = Events.Event(self._context,code)
            if bindings.has_key(event):
                widget.updateView(bindings[event])
            else:
                widget.updateView(None)


class EventWidget(Node.Node):

    def __init__(self, code, label, x, y, size=9):
        Node.Node.__init__(self)
        self.x, self.y, self.size = x, y, size
        self._win = None
        self.code = code
        self.label = label

    def init(self):
        self._win = self.owner.win().derwin(2, self.size, self.y, self.x)
        self._win.hline(0,0,curses.ACS_HLINE,self.size-1)
        self._win.addstr(0,(self.size-1-min(self.size-3,len(self.label)))/2, self.label[:self.size-3])
        self._win.nooutrefresh()

    def updateView(self, binding):
        if binding:
            self._win.addstr(1,0, binding.label[:self.size-1].center(self.size-1), curses.A_BOLD)
        else:
            self._win.addstr(1, 0, ' '*(self.size-1))
        self._win.nooutrefresh()


class ViewManager(object):

    _views = Node.NodeMap()

    def __init__(self, keylist, dispatcher, win):
        self._win = win
        keylist.registerCallback(self.updateViews)
        dispatcher.registerCallback(self.beforeIdle)

    def win(self):
        return self._win

    def registerView(self, view):
        self._views[view._context] = view
        view.init()

    def updateViews(self, keylist):
        bindings = keylist.bindingsByContext()
        for context, keys in bindings.iteritems():
            if self._views.has_key(context):
                self._views[context].updateView(keys)

    def beforeIdle(self):
        self._win.refresh()
